#ifndef ELOG_ENTRY_H
#define ELOG_ENTRY_H

#include <QDateTime>
#include <QPair>

class ElogEntry : public QObject
{
    Q_OBJECT

    Q_PROPERTY( QString   tech      READ getTech      WRITE setTech      NOTIFY techChanged      )
    Q_PROPERTY( QString   comment   READ getComment   WRITE setComment   NOTIFY commentChanged   )
    Q_PROPERTY( QDateTime timestamp READ getTimestamp WRITE setTimestamp NOTIFY timestampChanged )

public:
    explicit ElogEntry(const QDateTime & timestamp,
                       const QString & tech,
                       QVector<QPair<QString, QString> *> * const vars,
                       const QString & comment,
                       QObject *parent = 0)
        : QObject(parent)
        , m_tech(tech)
        , m_comment(comment)
        , m_timestamp(timestamp)
        , m_vars(vars)
    {
    }

    QString getTech()      const { return this->m_tech;      }
    QString getComment()   const { return this->m_comment;   }
    QDateTime getTimestamp() const { return this->m_timestamp; }
    QVector<QPair<QString, QString> *> * getArgs() const { return m_vars; }

signals:
    void techChanged();
    void commentChanged();
    void timestampChanged();


public slots:
    void setTech(const QString & tech)             { this->m_tech      = tech;      emit techChanged();     }
    void setComment(const QString & comment)       { this->m_comment   = comment;   emit commentChanged();  }
    void setTimestamp(const QDateTime & timestamp) { this->m_timestamp = timestamp; emit timestampChanged(); }

private:

    QString m_tech;
    QString m_comment;
    QDateTime m_timestamp;
    QVector<QPair<QString, QString> *> * m_vars;
};

#endif // ELOG_ENTRY_H

