/*
   08/28/14 - jas - modified to handle different formats of the date/time string
                    - the format statement used was not correct because the hour
                    element doesn't always have leading 0 - also modified to 
                    skip entries within the log file that did not contain the
                    corresponding header row
*/
#define XLIB_ILLEGAL_ACCESS

#include <QtSql>
#include <QFile>
#include <QStack>
#include <QString>
#include <QVector>
#include <iostream>
#include <QtArg/Arg>
#include <QtArg/Help>
#include <QMainWindow>
#include <QSqlDatabase>
#include <QtArg/XorArg>
#include <QtArg/CmdLine>
#include <QtGui/QApplication>

#include <iostream>
#include <sstream>
using namespace std;

#include "elog_entry.h"
unsigned int processingDelayValue = 0;

void spinner(int entries_processed)
{
    QString slash[4];
    slash[0] = "\\";
    slash[1] = "-";
    slash[2] = "/";
    slash[3] = "|";

    cout << "\r" << "reading logfile, entries processed: " << entries_processed << " " << slash[entries_processed % 4].toLocal8Bit().constData();
    cout.flush();
}

QStack<ElogEntry *> parseElog(const QString & filepath)
{
    size_t lineCount = 0;
    QStack<ElogEntry *> logbookStack;
    QString logbookPath;
    QString currentDir = QDir::currentPath();
    if(QFile(filepath).exists())
    {
        logbookPath = filepath;
    }
    else if(QFile(currentDir + "/" + filepath).exists())
    {
        logbookPath = QDir::currentPath() + filepath;
    }
    else
    {
        std::cerr << "Logbook file not found\n";
        exit(-1);
    }

    QFile logbook(logbookPath);
    logbook.open(QIODevice::ReadOnly);

    QTextStream elbStream(&logbook);

    /*
     * First five lines are always
     * ------------------------------
     * TEST CELL 108        LOG BOOK
     * ------------------------------
     *
     * ---------------------------------------------------------------------------
     * We don't care about them.
     */

    elbStream.readLine();
    ++lineCount;
    elbStream.readLine();
    ++lineCount;
    elbStream.readLine();
    ++lineCount;
    elbStream.readLine();
    ++lineCount;
    elbStream.readLine();
    ++lineCount;

    int x = 0;
    while (elbStream.atEnd() == false)
    {
       spinner(x++);

        /*
         * Next line should look like:
         *       Date      Time     HrMtr     EngHr    TestHr  Odometer PAM_point     Tech
         */

        QString entryline = elbStream.readLine();
        ++lineCount;

        while((entryline.isEmpty() || entryline.contains("      Date      Time") == 0 ) && elbStream.atEnd() == false )
        {
            entryline = elbStream.readLine();
            ++lineCount;
        }

        if(elbStream.atEnd() == true)
        {
            break;
        }

        QString valuesline = elbStream.readLine();
        ++lineCount;
        QString timestampStr = valuesline.mid(0, 20).replace(",'", " 20").trimmed();

        QStringList date_parts = timestampStr.split( " ", QString::SkipEmptyParts );

        QDate date_val = QDate::fromString( QString( "%1 %2 %3 %4")
                                               .arg( date_parts.at(0) )
                                               .arg( date_parts.at(1) )
                                               .arg( date_parts.at(2) )
                                               .arg( date_parts.at(3) ), "ddd MMM d yyyy" );

        QTime time_val = QTime::fromString( QString("%1").arg(date_parts.at(4)),
                                            "h:mm");

        QDateTime timestamp = QDateTime( date_val, time_val );

        if(! timestamp.isValid())
        {
            timestamp = QDateTime::fromString(timestampStr.trimmed(), "ddd MMM dd yyyy");
        }

        QString techname = "";

        QVector<QPair<QString, QString> *> * engVars = new QVector<QPair<QString, QString> *>();

        QStringList entryLineList  = entryline.mid(20).split(QRegExp("\\s+"), QString::SkipEmptyParts);
        QStringList valuesLineList = valuesline.mid(20).split(QRegExp("\\s+"), QString::SkipEmptyParts);

        int minCount = (entryLineList.count() < valuesLineList.count() ? entryLineList.count() : valuesLineList.count());
        for(int i = 0; i < minCount-1; ++i)
        {
            if(entryLineList[i].compare("Tech") == 0)
            {
                techname = valuesLineList[i];
                continue;
            }
            engVars->append(new QPair<QString,QString>(entryLineList[i], valuesLineList[i]));
        }

        if(minCount == 0)
        {
            cout << "Problem detected on line: " << lineCount << "\n";
        }
        else if(entryLineList[minCount-1].compare("Tech") == 0)
        {
            techname = valuesLineList[minCount-1];
        }
        else
        {
            QString lastEntry("");
            for(int i = minCount-1; i < entryLineList.count(); ++i)
            {
                lastEntry += entryLineList[i] + " ";
            }
            lastEntry = lastEntry.trimmed();

            QString lastValue("");
            for(int i = minCount-1; i < valuesLineList.count(); ++i)
            {
                lastValue += valuesLineList[i] + " ";
            }
            lastValue = lastValue.trimmed();


            engVars->append(new QPair<QString,QString>(lastEntry, lastValue));
        }

        QString restOfEntry;

        /*
         * Scan through the file looking for the end of the entry
         */
        QString tempLine = elbStream.readLine();
        while(tempLine.compare("---------------------------------------------------------------------------") != 0 && elbStream.atEnd() == false)
        {
            restOfEntry.append(tempLine).append("\n");
            tempLine = elbStream.readLine();
            ++lineCount;
        }

        logbookStack.push(new ElogEntry(timestamp, techname, engVars, restOfEntry.trimmed()));

        if(processingDelayValue > 0)
        {
           sleep(processingDelayValue);
        }
    }

    cout << "\n";
    return logbookStack;
}

typedef QPair<QString, QString> StringPair;

void progressbar(double progress, double total)
{

    static QString bars;
    static int x = 0;
    static int lastProgress = 0.0;
    double doublepercent = (progress/total);
    doublepercent *= 100.0;


    int percent = (int)doublepercent;

    QString slash[4];
    slash[0] = "\\";
    slash[1] = "-";
    slash[2] = "/";
    slash[3] = "|";
    if(lastProgress < percent)
    {
        bars.append('|');
        lastProgress = percent;
    }
    cout << "\r" << progress << "/" << total << " " << bars.toLocal8Bit().constData() << " " << slash[x % 4].toLocal8Bit().constData() << " " << percent << "%";
    ++x;
    cout.flush();
}

int main(int argc, char *argv[])
{

    QtArgCmdLine cmd( argc, argv );

    QtArg elogDB(QChar('e'),
                 QtArgIface::NamesList() << "elogdb",
                 "File path of the elog database to convert to.",
                 true,
                 true );
    elogDB.setLongDescription("The filepath of the elog database to put the existing elog entries into. This should probably match an entry in /etc/odbc.ini.");


    QtArg logfile(QChar('l'),
                  QtArgIface::NamesList() << "logfile",
                  "File path to the existing elog file.",
                  true,
                  true );
    logfile.setLongDescription("File path to the existing elog file to convert into the new database format.");

    QtArg processingDelay(QChar('d'),
                  QtArgIface::NamesList() << "delay",
                  "A delay, in milliseconds, between each iteration of processing the log file.",
                  false,
                  true );
    processingDelay.setLongDescription("A delay, in milliseconds, between each iteration of processing the log file.");

    QtArgHelp help( &cmd );
    help.printer()->setProgramDescription( "The convert_elog utility provides a commandline interface for converting from the old electonic log book format to the new database format."
                                           "\n\n"
                                           "For more help with a specific command, use --help <command>");
    help.printer()->setExecutableName( argv[0] );

    cmd.addArg(elogDB);
    cmd.addArg(logfile);
    cmd.addArg(processingDelay);
    cmd.addArg(help);

    try
    {
        cmd.parse();
    }
    catch( const QtArgHelpHasPrintedEx & x )
    {
        std::cout << x.what() << "\r\n";
        return -1;
    }
    catch( const QtArgBaseException & x )
    {
        std::cout << x.what() << "\r\n";
        return -1;
    }

    if(processingDelay.isPresent())
    {
       bool ok;
       processingDelayValue = processingDelay.value().toUInt(&ok);
       if(false == ok)
       {
          std::cout << "The value for the processing delay argument must be a number " << "\r\n";
          return -1;
       }
    }

    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
    db.setHostName("localhost");
    db.setDatabaseName(elogDB.value().toString());
    db.open();

    QSqlQuery query;

    query.exec("PRAGMA synchronous = OFF");
    query.exec("PRAGMA journal_mode = MEMORY");
    /*
     * Initialize the DB, if it has not already been done.
     */
    query.exec("CREATE TABLE IF NOT EXISTS elogbook (logID INTEGER PRIMARY KEY AUTOINCREMENT, timestamp DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP, comment TEXT NOT NULL, technician TEXT NOT NULL, elog_flag INTEGER NOT NULL, sentToMasterDB INTEGER NOT NULL DEFAULT 0);");
    query.exec("CREATE TABLE IF NOT EXISTS logbook_variables (logEntry INTEGER NOT NULL, label TEXT NOT NULL, value TEXT NOT NULL, labelOrder INTEGER NOT NULL, FOREIGN KEY(logEntry) REFERENCES elogbook(logID));");
    query.exec("CREATE TABLE IF NOT EXISTS engine_specification_variables (logEntry INTEGER NOT NULL, label TEXT NOT NULL, value TEXT NOT NULL, labelOrder INTEGER NOT NULL, FOREIGN KEY(logEntry) REFERENCES elogbook(logID));");
    query.exec("CREATE TABLE IF NOT EXISTS pam_variables (logEntry INTEGER PRIMARY KEY NOT NULL, Test_ID TEXT NOT NULL, Test_ID_extn TEXT NOT NULL, prog_proj INTEGER NOT NULL, FOREIGN KEY(logEntry) REFERENCES elogbook(logID));");

    QSqlQuery begin_transaction;
    begin_transaction.prepare("BEGIN TRANSACTION");
    QSqlQuery commit_transaction;
    commit_transaction.prepare("COMMIT TRANSACTION");
    /*
     * Parse the existing elog file.
     */
    QStack<ElogEntry *> entries = parseElog(logfile.value().toString());

    query.prepare("INSERT INTO elogbook (timestamp, comment, technician, elog_flag) VALUES (DATETIME(:timestamp, 'unixepoch'), :comment, :technician, :elog_flags);");

    QSqlQuery query2;
    query2.prepare("INSERT INTO logbook_variables (logEntry, label, value, labelOrder) VALUES ((SELECT MAX(rowid) FROM elogbook), :label, :value, (SELECT IFNULL(MAX(labelOrder),0)+1 FROM logbook_variables WHERE logEntry = (SELECT MAX(rowid) FROM elogbook)))");

    int size = entries.size();
    for(int i = 0; i < size; ++i)
    {
        begin_transaction.exec();
        progressbar(i,size);
        ElogEntry * entry = entries.pop();

        query.bindValue(":timestamp", entry->getTimestamp().toTime_t());
        query.bindValue(":comment", entry->getComment());
        query.bindValue(":technician", entry->getTech());
        query.bindValue(":elog_flags", "0");
        query.exec();

        foreach(StringPair * param, *(entry->getArgs()))
        {
            query2.bindValue(":label", ((param->first == "")  ? "          " : param->first));
            query2.bindValue(":value", ((param->second == "") ? "          " : param->second));
            query2.exec();
        }
        if(processingDelayValue > 0)
        {
           sleep(processingDelayValue);
        }
        commit_transaction.exec();
    }

    cout << endl;
    cout.flush();

    query.exec("PRAGMA synchronous = FULL");
    query.exec("PRAGMA journal_mode = WAL");
    return 0;
}
